/*****************************************************************************
** FILE IDENTIFICATION
**
**   Name:          projections.h
**   Purpose:       Header file for Projections class
**   Programmer:    Kevin Rosenberg
**   Date Started:  July 1, 1984
**
**  This is part of the CTSim program
**  Copyright (C) 1983-2000 Kevin Rosenberg
**
**  $Id: projections.h,v 1.2 2000/06/19 17:58:20 kevin Exp $
**
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License (version 2) as
**  published by the Free Software Foundation.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************/

#ifndef PROJECTIONS_H
#define PROJECTIONS_H

class Scanner;
class DetectorArray;

// Projections
class Projections
{
 public:
  Projections (const int nView, const int nDet);
  Projections (const Scanner& scanner);
  Projections (void);
  ~Projections (void);

  void initFromScanner (const Scanner& scanner);

  void printProjectionData (void);
  void printScanInfo (void) const;

  bool read (const char* fname);
  bool write (const char* fname);
  bool detarrayRead (DetectorArray& darray, const int view_num);
  bool detarrayWrite (const DetectorArray& darray, const int view_num);

  void setNView (int nView);  // used in MPI to restrict # of views
  void setRotInc (double rotInc) { m_rotInc = rotInc;}
  void setDetInc (double detInc) { m_detInc = detInc;}
  void setPhmLen (double phmLen) { m_phmLen = phmLen;}
  void setCalcTime (double calcTime) {m_calcTime = calcTime;}
  void setRemark (const char* remark) {m_remark = remark;}
  void setRemark (const string remark) {m_remark = remark;}

  const double detStart(void) const {return m_detStart;}
  const double rotStart(void) const {return m_rotStart;}
  const double calcTime(void) const {return m_calcTime;}
  const double phmLen(void) const {return m_phmLen;}
  const char*  remark(void) const {return m_remark.c_str();}
  const double detInc(void) const {return m_detInc;}
  const double rotInc(void) const {return m_rotInc;}
  const int nDet(void) const {return m_nDet;}
  const int nView(void) const {return m_nView;}
  DetectorArray& getDetectorArray (const int iview)
      { return (*m_projData[iview]); }
  
 private:
  int m_fd;			// Disk file descriptor 
  int m_headerSize;		// Size of disk file header 
  int m_geometry;		// Geometry of scanner 
  struct DetectorArray **m_projData;	// Pointer to array of detarray_st pointers 
  string m_remark;	        // description of raysum data 
  int m_nDet;			// number of detectors in array 
  int m_nView;			// number of rotated views 
  double m_calcTime;		// time required to calculate raysums 
  double m_rotStart;		// starting view rotation
  double m_rotInc;		// angle between rotations 
  double m_detStart;		// distance of beginning detector to center phantom
  double m_detInc;		// increment between detectors 
  double m_phmLen;		// Length of phantom edge (phm is square) 

  bool headerRead (void);
  bool headerWrite (void);
  void newProjData (void);
  void deleteProjData (void);

  void init (const int nView, const int nDet);
};


#endif
