/*****************************************************************************
**  This is part of the CTSim program
**  Copyright (C) 1983-2000 Kevin Rosenberg
**
**  $Id: sgp.h,v 1.10 2000/06/19 19:04:05 kevin Exp $
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License (version 2) as
**  published by the Free Software Foundation.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************/

/*----------------------------------------------------------------------*/
/*		   Standard Graphics Package Header File		*/
/*----------------------------------------------------------------------*/

#ifndef __H_SGP
#define __H_SGP

#if HAVE_CONFIG_H
#include "config.h"
#endif

#include "ctsupport.h"

#if HAVE_G2_H
extern "C" {
#include "g2.h"
#include "g2_X11.h"
}
#endif

/* device names */

#define CRTDEV  1
#define PRTDEV  2
#define MEMDEV  4
#define FILEDEV 8

/* linestyles */

#define LS_NOLINE 0
#define LS_SOLID  0xffff
#define LS_DASH1  0xff00
#define LS_DASH2  0xf0f0
#define LS_DASH3  0xcccc
#define LS_DASH4  0xff3e
#define LS_DOTTED 0xaaaa

#define MAXDASH	   4
#define MAXCOLOR   63

/* data structures */

struct sgp_window_st {
    int pw_xsize;         /* Physical Window size */
   int pw_ysize;
   char title[256];
   int g2_id;
    double xw_min;    /* Window extents */
    double yw_min;
    double xw_max;
    double yw_max;
    double xv_min;    /* Viewport extents */
    double yv_min;
    double xv_max;
    double yv_max;
    double view[4];   /* Viewport array for clip_rect() */
    double curx;      /* Current stylus position in world coords */
    double cury;
    int recalc_mc_to_ndc;
    int recalc_ndc_to_mc;
    GRFMTX_2D wc_to_ndc_x;     /* World coord to NDC matrix */
    GRFMTX_2D mc_to_ndc_x;     /* Master to NDC */
    GRFMTX_2D ndc_to_mc_x;     /* NDC to Master */
    GRFMTX_2D ctm_2_x;	       /* Current transfromation matrix */

    int phys_curx;
    int phys_cury;
};

typedef struct sgp_window_st SGP_WINDOW;
typedef struct sgp_window_st *SGP_ID;

struct device_st {
	int open;			/* TRUE if device is open for output */
	int xsize, ysize;		/* Size of device in pixels */
	int xmin, ymin;			/* smallest coordinates */
	int xmax, ymax;			/* Maximum coordinates */
	int colormax;			/* Maximum color number of device */
	int style;			/* Current linestyle of device */
	int width;			/* Current width of device */
	int color;			/* Current color of device */
	int icurx, icury;		/* Current position */
	int icwidth, icheight;		/* Size of characters in pixels */
	int cfore, cback;		/* Character foregnd & backgnd colors */
	float asp;			/* Aspect ratio.  Multipy x coord */
	int (*dotfunc)(int x1, int y1, int color);		/* Dot function for device */
	int (*linefunc)(int x1, int y1, int x2, int y2, int color);		/* Line function for device */
	unsigned int nbytes;		/* Size of buffer in bytes */
	char *buf;			/* Pointer to buffer */
	unsigned int bufseg, bufoff;	/* Buffer memory location */
	int mode;			/* Device mode */
};

struct charsp_st {
	float width, height;		/* size of characters in NDC */
	float textangle;		/* text angle in radians */
	float charupangle;		/* character up angle */
	int font;			/* font for characters */
	int updir, textdir;		/* text direct & character orientation */
	int fore, back;			/* foreground & background color */
					/* if back = -1, then transparent back */
};

struct state_st {
	int foregnd, backgnd;		/* current foregound & background colors */
	int linestyle;			/* current 16 bit linestyle */
	int linewidth;			/* current width of line (in pixels) */
	int marktype;			/* current marker type */
	int markcolor;			/* current marker color */
	float xndc, yndc;		/* current position in NDC */
};

typedef struct device_st DEVICE;
typedef struct charsp_st CHARSPEC;
typedef struct state_st GRFSTATE;

struct point	{double x, y, z;};


/* Constants */

/* Flagcodes for motion directions */
#define XPLUS	001	/* right */
#define XMINUS	002	/* left */
#define YPLUS	004	/* up */
#define YMINUS	010	/* down */

/* Codes for marker symbols */
#define POINT	 0	/* small dot */
#define SQUARE	 1	/* empty square */
#define FSQUARE  2	/* filled square */
#define DIAMOND  3	/* empty diamond */
#define FDIAMOND 4	/* filled diamond */
#define CROSS	 5	/* cross */
#define XCROSS	 6	/* x */
#define CERCLE	 7	/* open circle */
#define FCERCLE  8	/* filled circle */
#define BSQUARE  9	/* big open square */
#define BDIAMOND 10	/* big open diamond */

#define NMARKERS 11	/* Number of available symbol types */

/*-------------------------------------------------------------------------*/

#define PSET   0		/* codes for raster merging */
#define PRESET 1
#define OR     2
#define AND    3
#define XOR    4

/*-------------------------------------------------------------------------*/

/* circle.cpp */
void sgp2_draw_circle(const double r);
void sgp2_draw_arc(double start, double stop, const double r);

/* ctm.cpp */
void ctm_xlat_pre_2(double x, double y);
void ctm_xlat_post_2(double x, double y);
void ctm_scale_pre_2(double sx, double sy);
void ctm_scale_post_2(double sx, double sy);
void ctm_rotate_pre_2(double theta);
void ctm_rotate_post_2(double theta);
void ctm_shear_pre_2(double shrx, double shry);
void ctm_shear_post_2(double shrx, double shry);
void xlat_gmtx_2(GRFMTX_2D m, double x, double y);
void scale_gmtx_2(GRFMTX_2D m, double sx, double sy);
void shear_gmtx_2(GRFMTX_2D m, double shrx, double shry);
void rotate_gmtx_2(GRFMTX_2D m, double theta);
void ident_gmtx_2(GRFMTX_2D m);
void mult_gmtx_2(GRFMTX_2D a, GRFMTX_2D b, GRFMTX_2D c);
void invert_gmtx_2(GRFMTX_2D a, GRFMTX_2D b);
double determ_gmtx_2(GRFMTX_2D a);

/* drawbox.cpp */
void sgp2_draw_rect (double xmin, double ymin, double xmax, double ymax);

/* sgp.cpp */
SGP_ID sgp2_init (int xsize, int ysize, const char *title);
void sgp2_close (SGP_ID gid);
void sgp2_set_active_win (SGP_ID);
SGP_ID sgp2_get_active_win (void);
void sgp2_clear (void);
void sgp2_window(double xmin, double ymin, double xmax, double ymax);
void sgp2_viewport(double xmin, double ymin, double xmax, double ymax);
void sgp2_frame_vpt(void);
void calc_wc_to_ndc(void);
void calc_ndc_to_mc(void);
void wc_to_ndc(double xw, double yw, double *xn, double *yn);
void ndc_to_wc(double xn, double yn, double *xw, double *yw);
void sgp2_color(int icol);
void sgp2_line_style(int style);
void sgp2_line_abs(double x, double y);
void sgp2_move_abs(double x, double y);
void sgp2_line_rel(double x, double y);
void sgp2_move_rel(double x, double y);
void sgp2_draw_text(char *message);
void sgp2_polyline_abs(double x[], double y[], int n);
void sgp2_mark_abs(double x, double y);
void sgp2_mark_rel(double x, double y);
void sgp2_point_abs(double x, double y);
void sgp2_point_rel(double x, double y);
void charsize(double wid, double height);
void textangle(double angle);
void ctm_clr_2(void);
void ctm_get_2(GRFMTX_2D m);
void ctm_set_2(GRFMTX_2D m);
void ctm_pre_mult_2(GRFMTX_2D m);
void ctm_post_mult_2(GRFMTX_2D m);

/* sgpdrive.cpp */
int _sgp2_init_dev(SGP_ID gid);
int initdevice(int dev, int mode, int xsize, int ysize);
int opendevice(int dev);
void closedevice(int dev);
void termdevice(int dev);
void _sgp2_stylus(SGP_ID gid, double x, double y, int beam);
void pntndc (SGP_ID gid, double x, double y);
void markndc (SGP_ID gid, double x, double y);
GRFSTATE *inqstate(void);
void gp_set_aspect(int dev, double asp);
void setlinestyle(int style);
void setlinewidth(int wid);
DEVICE *inqdev(int dev);
void _sgp2_set_text(SGP_ID gid, double width, double height, double textangle, int font);
void settextclr(int fore, int back);
void setcolor(int fore);
void setbackg(int back);
int initmarker(int marker, int color);
int settextdir(int direction);
void _sgp2_dev_text(SGP_ID gid, char *message);
void termgrf2(void);
void flushdevice(int dev);

/* sgptext.cpp */
void wrtsymbol(int sym, int x, int y, DEVICE *dev);
void wrtchar(int ch, int x, int y, CHARSPEC *cspec, DEVICE *dev);
void wrttext(char txtstr[], int x, int y, CHARSPEC *cspec, DEVICE *dev);
void crtcolor(int mode, int *f, int *b);


#endif
