/*****************************************************************************
** FILE IDENTIFICATION
**
**	Name:	      imagefile.cpp
**      Purpose:      Imagefile classes
**	Programmer:   Kevin Rosenberg
**	Date Started: June 2000
**
**  This is part of the CTSim program
**  Copyright (C) 1983-2000 Kevin Rosenberg
**
**  $Id: imagefile.cpp,v 1.1 2000/06/19 02:59:34 kevin Exp $
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License (version 2) as
**  published by the Free Software Foundation.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************/

#include "ct.h"


///////////////////////////////////////////////////////////////////////////
// CLASS IMPLEMENTATION
//
//     Name: Array2dFileLabel
//     Purpose: Labels for Array2dFiles
///////////////////////////////////////////////////////////////////////////

void
Array2dFileLabel::init (void)
{
    m_calcTime = 0;
    m_labelType = L_EMPTY;
    TIMEDATE td;
    td_get_tmdt (&td);
    m_year = td.d.year;
    m_month = td.d.month;
    m_day = td.d.date;
    m_hour = td.t.hour;
    m_minute = td.t.minute;
    m_second = td.t.second;
}

Array2dFileLabel::Array2dFileLabel() 
{
    init();
}

Array2dFileLabel::Array2dFileLabel(const char* const str, double ctime = 0.)
  : m_strLabel (str)
{
    init();

    m_labelType = L_USER;
    m_calcTime = ctime;
}

Array2dFileLabel::Array2dFileLabel(const int type, const char* const str, double ctime = 0.)
  :  m_strLabel (str)
{
    init();

    m_labelType = type;
    m_calcTime = ctime;
}

Array2dFileLabel::~Array2dFileLabel()
{
}

const string& 
Array2dFileLabel::getDateString (void) const
{
  ostringstream oss;
  oss <<  m_month <<"/"<< m_day <<"/"<< m_year << " " << m_hour <<":"<<  m_minute <<":"<< m_second;
  m_strDate = oss.str();
  return m_strDate;
}


/* FILE
 *   image.c				Routines for managing images
 *
 * PROGRAMMER:	Kevin Rosenberg
 * DATE:	Aug 1984
 *
 * FUNCTION
 *	Provides a set of routines for handling image files
 */


void 
image_filter_response (ImageFile& im, const DomainType domain, double bw, const FilterType filt_type, double filt_param, const int opt_trace)
{
  int hx = im.nx() / 2;
  int hy = im.ny() / 2;
  ImageFileArray v = im.getArray();

  for (int i = -hx; i <= hx; i++) {
    for (int j = -hy; j <= hy; j++) {
      double r = sqrt(i * i + j * j);
      
      if (domain == D_SPATIAL)
	v[i+hx][j+hy] = filter_spatial_response_analytic (filt_type, r, bw, filt_param);
      else
	v[i+hx][j+hy] = filter_frequency_response (filt_type, r, bw, filt_param);
      if (opt_trace >= TRACE_PHM)
	printf ("r=%8.4f, v=%8.4f\n", r, v[i+hx][j+hy]);
    }
  }
}

int
image_display (const ImageFile& im)
{
    ImageFileValue pmin, pmax;

    im.getPixelValueRange (pmin, pmax);

    return (image_display_scale (im, 1, pmin, pmax));
}

int image_display_scale (const ImageFile& im, const int scale, const double pmin, const double pmax)
{
    int grayscale[256];
    int nx = im.nx();
    int ny = im.ny();
    ImageFileArray v = im.getArray();

#if HAVE_G2_H
    int* pens = new int [nx * ny * scale * scale ];

    double view_scale = 255 / (pmax - pmin);
    int id_X11 = g2_open_X11 (nx * scale, ny * scale);

    for (int i = 0; i < 256; i++) {
	double cval = i / 255.;
	grayscale[i] = g2_ink (id_X11, cval, cval, cval);
    }

    for (int i= 0, iy = ny - 1; iy >= 0; iy--) {
	for (int ix = 0; ix < nx; ix++) {
	    int cval = (int) ((v[ix][iy] - pmin) * view_scale);
	    if (cval < 0)  
	      cval = 0;
	    else if (cval > 255) 
	      cval = 255;
	    pens[i++] = grayscale[cval];
	}
    }

    g2_image (id_X11, 0., 0., nx, ny, pens);

    delete [] pens;
    return (id_X11);

#endif
}




